var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('oldZhCalendar', function(ZhUtil, ScreenService){
	return {
		restrict: 'A',
		link: function($scope, element) {

			var controlFormat = "YYYY-MM-DD";
			$scope.dateFormat = $scope.widget.dateFormat || "L";

			var widget = $scope.widget;

			$scope.today = moment().startOf('day');

			// Verify the if the calendar is a field or an widget
			if(widget.field){
				if (widget.field.currentDate) {
					$scope.currentDate = moment(widget.field.currentDate, widget.field.dateFormat);
				} else if (widget.field.defaultValue) {
					$scope.currentDate = moment(ZhUtil.evaluate(widget.field.defaultValue), $scope.dateFormat).startOf('month').startOf('day');
				}
			} else{
				if (widget.currentDate) {
					$scope.currentDate = moment(widget.currentDate, $scope.dateFormat).startOf('month').startOf('day');
				}
			}

			if (!$scope.currentDate || !$scope.currentDate.isValid()) {
				$scope.currentDate = moment().startOf('month').startOf('day');
			}

			$scope.todayText = $scope.widget.todayText || ScreenService.i18n("Today");
			$scope.weekDays = $scope.widget.weekDays || moment.weekdaysShort();
			$scope.monthNames = $scope.widget.monthNames || moment.months();

			$scope.maxDate = "";
			$scope.momentMaxDate = "";

			$scope.minDate = "";
			$scope.momentMinDate = "";

			function updateMaxMinDate() {
				$scope.maxDate = $scope.widget.maxDate === "today" ? moment().format($scope.dateFormat) : $scope.widget.maxDate;
				$scope.momentMaxDate = moment($scope.maxDate, $scope.dateFormat);

				$scope.minDate = $scope.widget.minDate === "today" ? moment().format($scope.dateFormat) : $scope.widget.minDate;
				$scope.momentMinDate = moment($scope.minDate, $scope.dateFormat);
			}
			updateMaxMinDate();

			$scope.isBlockedDate = function(date){
				return (($scope.momentMinDate.isValid() && moment(date, $scope.dateFormat).isBefore($scope.momentMinDate)) || ($scope.momentMaxDate.isValid() && moment(date, $scope.dateFormat).isAfter($scope.momentMaxDate)));
			};

			function buildCalendarData() {
				// Clones the current date to prevent changing it
				var current = $scope.currentDate.clone().utc().startOf('week').day(0);

				// Checks if the first day of the month is also the first day of its week so we should put it on the second row of the calendar
				if (current.format(controlFormat) === $scope.currentDate.clone().format(controlFormat)) {
					current.subtract(7, 'day');
				}

				// Iterates through the datasource seeking for this month events and builds an object grouping them
				var monthEvents = {};
				if ($scope.widget.dataSource && $scope.widget.dataSource.data) {
					$scope.widget.dataSource.data
						.filter(function(item) {
							var month = moment(item[$scope.widget.dateField], $scope.dateFormat).get('month');
							var currMonth = current.get('month');
							var isPreviousMonth = month === currMonth;
							var isCurrentMonth = month === ((currMonth + 1) % 12);
							var isNextMonth = month === ((currMonth + 2) % 12);
							return isPreviousMonth || isCurrentMonth || isNextMonth;
						})
						.forEach(function(item) {
							var itemTime = moment(item[$scope.widget.dateField], $scope.dateFormat).startOf('day').format(controlFormat);

							if (!monthEvents[itemTime]) {
								monthEvents[itemTime] = [];
							}

							monthEvents[itemTime].push(item);
						});
				}

				// Start creating week
				$scope.weeks = [];
				for (var i = 0; i < 6; i++) {
					var week = {
						days: []
					};
					for (var j = 0; j < 7; j++) {
						week.days.push({
							day: current.date(),
							date: current.format($scope.dateFormat),
							today: current.format(controlFormat) === $scope.today.format(controlFormat),
							lastMonth: (current.month() < $scope.currentDate.month() && current.year() == $scope.currentDate.year()) || (current.month() > $scope.currentDate.month() && current.year() < $scope.currentDate.year()),
							nextMonth: (current.month() > $scope.currentDate.month() && current.year() == $scope.currentDate.year()) || (current.month() < $scope.currentDate.month() && current.year() > $scope.currentDate.year()),
							events: monthEvents[current.format(controlFormat)] || [],
							selected: false
						});
						current.add(1, 'day');
					}
					$scope.weeks.push(week);
				}
			}

			// Navigation methods
			$scope.navigateDate = function(value, unit) {
				var oldYear = $scope.currentDate.year();
				$scope.currentDate.add(value, unit);
				switch (unit){
					case "month":
						handleEventCurrentMonthChange();
						if(oldYear !== $scope.currentDate.year()){
							handleEventCurrentYearChange();
						}
						break;
					case "year":
						handleEventCurrentYearChange();
						break;
				}
			};

			$scope.goToday = function() {
				var today = $scope.today.clone().startOf('month');
				if (today.year() !== $scope.currentDate.year()) {
					handleEventCurrentYearChange();
				}
				if (today.month() !== $scope.currentDate.month()) {
					handleEventCurrentMonthChange();
				}
				$scope.currentDate = today;
			};

			$scope.widget.getMonth = function() {
				return  $scope.monthNames[$scope.currentDate.get('month')];
			};

			$scope.widget.getYear = function() {
				return $scope.currentDate.get('year');
			};

			function handleEventCurrentMonthChange(){
				if($scope.widget.onChangeMonth){
					$scope.widget.onChangeMonth();
				}
			}

			function handleEventCurrentYearChange(){
				if($scope.widget.onChangeYear){
					$scope.widget.onChangeYear();
				}
			}

			$scope.dateClick = function(day) {
				if(day.lastMonth) {
					$scope.navigateDate(-1, 'month');
				} else if(day.nextMonth) {
					$scope.navigateDate(+1, 'month');
				} else if(!$scope.isBlockedDate(day.date)){
					if ($scope.widget.dateClick) {
						$scope.widget.dateClick();
					}
					if($scope.widget.field){
						$scope.widget.field.currentDate = $scope.currentDate;
					}
					$scope.widget.setCurrentRow(day.date);
				}
			};

			$scope.$watch(
				function() {
					return $scope.widget.maxDate;
				},
				function() {
					if($scope.widget.maxDate){
						updateMaxMinDate();
					}
				}
			);

			$scope.$watch(
				function() {
					return $scope.widget.minDate;
				},
				function() {
					if($scope.widget.minDate){
						updateMaxMinDate();
					}
				}
			);

			/** @todo: Implement a multi-date selection
			 *
			 *	$scope.selectDate = function(dayObj) {
			 *		var fullDate = dayObj.date + '-' + dayObj.month + '-' + dayObj.year;
			 *		element.find("td[data-date=" + fullDate + "]").toggleClass('selected');
			 *	};
			 *
			 **/

			$scope.$watch(
				function() {
					return $scope.currentDate._d;
				},
				function() {
					$scope.currentMonth = $scope.monthNames[$scope.currentDate.get('month')];
					$scope.currentYear = $scope.currentDate.get('year');
					buildCalendarData();
				},
				true
			);

			$scope.$watch(
				function() {
					if($scope.widget.dataSource) {
						return $scope.widget.dataSource.data;
					}
				},
				function() {
					$scope.currentMonth = $scope.monthNames[$scope.currentDate.get('month')];
					$scope.currentYear = $scope.currentDate.get('year');
					buildCalendarData();
				}
			);

		}
	};
});